////////////////////////////////////////////////////////////////////////////////
//
//  IDEUM
//  Copyright 2011-2015 Ideum
//  All Rights Reserved.
//
//  Gestureworks Core
//
//  File:    HelloMultitouchApp.cpp
//  Authors:  Ideum
//
//  NOTICE: Ideum permits you to use, modify, and distribute this file only
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

#include "HelloMultitouchApp.h"
#include "GestureWorks.h"
#include "cinder/Rect.h"

std::pair<int, int> HelloMultitouchApp::normScreenToWindowPx(float screen_x, float screen_y){
	Vec2i rect = getWindowSize();

	int x_px;
	int y_px;
	if (!use_pixels)
	{
		x_px = static_cast<int>(screen_x * screen_width);
		y_px = static_cast<int>(screen_y * screen_height);
	}
	else
	{
		x_px = static_cast<int>(screen_x);
		y_px = static_cast<int>(screen_y);
	}

	int x = x_px - getWindowPosX();
	int y = y_px - getWindowPosY();

	return std::pair<int, int>(x, y);
}

void HelloMultitouchApp::prepareSettings(Settings *settings) {
	settings->setWindowSize(1024, 768);
	settings->setTitle("Hello Multitouch!");
	settings->setFrameRate(60.0f);

	screen_width = GetSystemMetrics(SM_CXSCREEN);
	screen_height = GetSystemMetrics(SM_CYSCREEN);
}

void HelloMultitouchApp::setup()
{
	// create font
	font = Font("Arial", 22);
	texture_font = gl::TextureFont::create(font);

	active_points = std::map<int, gwc::Point>();
	if (!GestureWorks::Instance()->LoadGestureWorks(L"GestureworksCore32.dll")) {
		console() << "Error loading gestureworks dll" << std::endl;
	}

	GestureWorks::Instance()->InitializeGestureWorks(0, 0);

	if (!GestureWorks::Instance()->RegisterWindowForTouchByName(L"Hello Multitouch!")) {
		console() << "Could not register target window for touch." << std::endl;
	}

	use_pixels = true;
	GestureWorks::Instance()->SetUsePixels(use_pixels);
}

void HelloMultitouchApp::mouseDown(MouseEvent event)
{
}

void HelloMultitouchApp::update()
{
	GestureWorks::Instance()->ProcessFrame();
	std::vector<gwc::PointEvent> point_events = GestureWorks::Instance()->ConsumePointEvents();

	for (std::vector<gwc::PointEvent>::iterator event_it = point_events.begin(); event_it != point_events.end(); event_it++) {

		std::pair<int, int> pair = normScreenToWindowPx(event_it->position.x, event_it->position.y);

		switch (event_it->status) {
		case gwc::TOUCHADDED:
		case gwc::TOUCHUPDATE:
			//If the point is being added, this will place it in our point map;
			//the same line of code will update the point's position if it's
			//already present, so we can use this command to handle new points
			//as well as point updates
			active_points[event_it->point_id] = gwc::Point(pair.first, pair.second);
			break;
		case gwc::TOUCHREMOVED:
			//Remove the point from the map
			active_points.erase(event_it->point_id);
			break;
		}
	}
}

void HelloMultitouchApp::draw()
{
	gl::clear(Color(0.66f, 0.66f, 0.66f), true);
	gl::enableAlphaBlending();
	gl::color(Color(1.0f, 0.2f, 0.0f));

	//For each active point
	for (std::map<int, gwc::Point>::iterator points_it = active_points.begin(); points_it != active_points.end(); points_it++)
	{
		float x = points_it->second.x;
		float y = points_it->second.y;


		//Generate a circle at this point's location
		gl::drawSolidCircle(Vec2f(x, y), 40.0f);
		gl::drawStrokedCircle(Vec2f(x, y), 50.0f);

		//Generate a stringstream for each value with which we're concerned
		std::stringstream xvals; xvals << x;
		std::stringstream yvals; yvals << y;
		std::stringstream ids; ids << points_it->first;

		//Annotate the circle we just drew with the id, x and y values for the corresponding point
		texture_font->drawString("ID: " + ids.str() + "    X: " + xvals.str() + "  " + " Y: " + yvals.str(), Vec2f(x + 40.0f, y - 40.0f));
	}
}

CINDER_APP_NATIVE(HelloMultitouchApp, RendererGl)
